@@ -1,5 +1,27 @@
 (* Because other the syntax s.[x] causes trouble *)
 module String = Stdlib.String
+
+module StringLabels = struct
+  (* functions potentially in the stdlib, depending on OCaml version *)
+
+  let[@warning "-32"] exists =
+    let rec loop s i len f =
+      if i = len then false else f (String.unsafe_get s i) || loop s (i + 1) len f
+    in
+    fun ~f s -> loop s 0 (String.length s) f
+  ;;
+
+  let[@warning "-32"] for_all =
+    let rec loop s i len f =
+      i = len || (f (String.unsafe_get s i) && loop s (i + 1) len f)
+    in
+    fun ~f s -> loop s 0 (String.length s) f
+  ;;
+
+  (* overwrite them with stdlib versions if available *)
+  include Stdlib.StringLabels
+end
+
 include StringLabels
 
 let compare a b = Ordering.of_int (String.compare a b)
@@ -180,18 +202,6 @@ let longest_prefix = function
     sub ~pos:0 x ~len:(loop len 0)
 ;;
 
-let exists =
-  let rec loop s i len f =
-    if i = len then false else f (unsafe_get s i) || loop s (i + 1) len f
-  in
-  fun s ~f -> loop s 0 (length s) f
-;;
-
-let for_all =
-  let rec loop s i len f = i = len || (f (unsafe_get s i) && loop s (i + 1) len f) in
-  fun s ~f -> loop s 0 (length s) f
-;;
-
 let quoted = Printf.sprintf "%S"
 
 let maybe_quoted s =
@@ -221,12 +231,6 @@ let enumerate_one_of = function
   | s -> "One of " ^ enumerate_or s
 ;;
 
-let concat ~sep = function
-  | [] -> ""
-  | [ x ] -> x
-  | xs -> concat ~sep xs
-;;
-
 let take s len = sub s ~pos:0 ~len:(min (length s) len)
 
 let drop s n =
@@ -274,7 +278,22 @@ let need_quoting s =
 ;;
 
 let quote_for_shell s = if need_quoting s then Stdlib.Filename.quote s else s
-let quote_list_for_shell l = List.map l ~f:quote_for_shell |> concat ~sep:" "
+
+let quote_list_for_shell = function
+  | [] -> ""
+  | prog :: args ->
+    let prog =
+      if Sys.win32 && contains prog '/'
+      then
+        map
+          ~f:(function
+            | '/' -> '\\'
+            | c -> c)
+          prog
+      else prog
+    in
+    quote_for_shell prog :: List.map ~f:quote_for_shell args |> concat ~sep:" "
+;;
 
 let of_list chars =
   let s = Bytes.make (List.length chars) '0' in
@@ -286,3 +305,24 @@ let filter_map t ~f =
   (* TODO more efficient implementation *)
   to_seq t |> Seq.filter_map ~f |> of_seq
 ;;
+
+let drop_prefix_and_suffix t ~prefix ~suffix =
+  let p_len = String.length prefix in
+  let s_len = String.length suffix in
+  let t_len = String.length t in
+  let p_s_len = p_len + s_len in
+  if p_s_len <= t_len && is_prefix t ~prefix && is_suffix t ~suffix
+  then Some (sub t ~pos:p_len ~len:(t_len - p_s_len))
+  else None
+;;
+
+let contains_double_underscore =
+  let rec aux s len i =
+    if i > len - 2
+    then false
+    else if s.[i] = '_' && s.[i + 1] = '_'
+    then true
+    else aux s len (i + 1)
+  in
+  fun s -> aux s (String.length s) 0
+;;
